/* Implementation of get_input.h */

#include "get_matlab.h"
#include "get_input.h"
/* Get date */

        
double *Get3DImage(const mxArray *prhs,ulng &nRows,ulng &nColumns,ulng &nColours)
{
    /* Get the image */
    double *m3DImage = Get3DVar(prhs,nRows,nColumns,nColours,"Image");
    
    return m3DImage;
    
}

double *GetVWeights(const mxArray *prhs, ulng nColours)
{
	double *vWeights = 0;    
    unsigned long nRows = 0;
    unsigned long nColumns = 0;
    
    vWeights = Get2DVar(prhs,nRows,nColumns,"Weights"); 
    
    ValidateVWeights(nColours,nRows,nColumns);
    
    return vWeights;    
}

double *GetDict(const mxArray *prhs, unsigned long nRowsToCheck,
        unsigned long &nAtoms)
{
    /* Get the dictionary */
    unsigned long nRows = 0;
    //unsigned long nColumn = 0;
    double *mDict = Get2DVar(prhs,nRows,nAtoms,
            "Dictionary");    
    
    /* Validate dimensions */
    ValidateDict(nRowsToCheck,nRows);
    
    return mDict;
    
}

double *GetVIndDictRow(const mxArray *prhs, ulng nIndColumn)
{
	double *vIndDict = 0;
	if ((mxIsEmpty(prhs)) && (nIndColumn == 0))
	{
		vIndDict = 0;
	}else{

		/* Get the initial indexes chosen from the column dictionary*/
		unsigned long nRows = 0;
		unsigned long nColumns = 0;
		vIndDict = Get2DVar(prhs,nRows,nColumns,"IndDictRow");  
    
		ValidateVIndDictR(nIndColumn,nRows,nColumns);
	}
    
    return vIndDict;
    
}

double *GetVIndDictColumn(const mxArray *prhs, ulng nAtoms, ulng &nInd)
{
	double *vIndDict  = 0;
	if (mxIsEmpty(prhs))
	{
		nInd = 0;
		vIndDict = 0;
	}else{

		/* Get the initial indexes chosen from the column dictionary*/
		unsigned long nRows = 0;
		unsigned long nColumns = 0;
		vIndDict = Get2DVar(prhs,nRows,nColumns,"IndDictColumn");  
    
		nInd = ValidateVIndDictC(nAtoms,nRows,nColumns);
	}
    
    return vIndDict;
    
}

/* Validate */
ulng ValidateVIndDictC(unsigned long nAtoms, unsigned long nRows,
        unsigned long nColumns)
{
    /* Checks that the number of indexes is less than or equal to the 
     * number of atoms */
    

    /* The largest dimension is the number of indexes */
    ulng nIndexes = ValidateVector(nRows, nColumns);
        
    
    /* Validate */
    if (nIndexes > nAtoms)
    {
        mexPrintf("# of atoms %i, # of indexes %i\n",
            nAtoms, nIndexes);
        mexErrMsgTxt("Number of indexes should be less than the number of atoms!!");                
    }
    
    return nIndexes;
}

void ValidateVIndDictR(unsigned long nIndColumn, unsigned long nRows,
        unsigned long nColumns)
{
    /* Checks that the number of indexes is less than or equal to the 
     * number of atoms */
    
    /* The largest dimension is the number of indexes */
    ulng nIndexes = ValidateVector(nRows, nColumns);
        
    
    /* Validate */
    if (nIndexes != nIndColumn)
    {
        mexPrintf("# of initial column indexes %i, # of intitial row indexes %i\n",
            nIndColumn, nIndexes);
        mexErrMsgTxt("Number of indexes should be the same!!");                
    }

}

void ValidateDict(unsigned long nRowsToCheck, unsigned long nRows)
{
    if (nRowsToCheck != nRows)
    {
        mexPrintf("Dictionary has %i rows, validate against %i rows\n",
                nRowsToCheck,nRows);
        mexErrMsgTxt("Error # rows in dictionary not consistent!!!!");
    }
    
}

void ValidateVWeights(ulng nColours, ulng nRows, ulng nColumns)
{
    /* Check that the largest dimension is equal to nColours
     */
    
    // Check the number of elements in the vector is nColours
    if (nColours != ValidateVector(nRows,nColumns))
    {
        mexErrMsgTxt("Number of weights not equal to size of third dimension!!!");
    }
    
    // Check weights sum to 1  FUTURE
    
}
