/* Implementation of matrix_operations.h */

#include "matrix_operations.h"
#include "my_blas.h"
#include <string.h>
#include <math.h>
#define BLAS

void Matrix::VOuter(dbl *matrix, dbl *v1, dbl *v2, ulng m,
        ulng n, dbl alpha)
{
	/* Performs the vector outer product matrix = alpha*v1*v2' */

	#ifdef BLAS    
		// Zero the return matrix
		ZeroFillDbl(matrix,m*n);

        OuterBLAS(matrix,v1,v2,alpha,m,n);
    #else
        
        
        for ( ulng j = 0; j < n ; j++ )
        {      
            /* alpha = (beta*D(:,k)')/nork; */     
            //alpha = RealInnerProduct(&mBiorthogonal[j*szSignal],vNewAtom,szSignal)/normAtom;
            for ( ulng i = 0; i < m; i++ )
            {	          	 	  
                matrix[(j*m)+i] =  alpha*v2[j]*v1[i];                
            }            

        }                
    
    #endif
}

void Matrix::VOuterG(dbl *matrix, dbl *v1, dbl *v2, ulng m,
        ulng n,dbl alpha)
{
	/* Performs y<-alpha*x*t' + A*/

	#ifdef BLAS    
        OuterBLAS(matrix,v1,v2,alpha,m,n);
    #else                
        for ( ulng j = 0; j < n ; j++ )
        {      
            /* alpha = (beta*D(:,k)')/nork; */     
            //alpha = RealInnerProduct(&mBiorthogonal[j*szSignal],vNewAtom,szSignal)/normAtom;
            for ( ulng i = 0; i < m; i++ )
            {	          	 	  
                matrix[(j*m)+i] =  matrix[(j*m)+i] + alpha*v2[j]*v1[i];                
            }            

        }                
    
    #endif
}

void CopyDbl(dbl *array1, dbl *array2, ulng nElements)
{    
    size_t szCopy;
    szCopy = sizeof(dbl)*nElements;
    memcpy(array2, array1, szCopy);
}


void Matrix::MVMult(dbl *m1, dbl *v1, dbl *returnVector, 
        ulng m, ulng n, ulng nRowsReturn, char chn, dbl alpha)
{
	/* Performs a matrix vector constant mult */
#ifdef BLAS
		// Sets the return vector to zero and beta to 1
		ZeroFillDbl(returnVector,nRowsReturn);
        MatrixVectorMultBLAS(m1, v1, returnVector, m, n, alpha, 1, chn);
#else
        if( chn == 'T' )
        {
            for ( ulng i = 0; i < n ; i++)
            {   
                returnVector[i] = alpha*RealInnerProduct(&m1[(i*m)],v1,m);
            }
        }else{            
            for ( ulng i = 0; i < m ; i++)
            {
                //returnVector[i] = 0;
				//returnVector[i] = beta*returnVector[i];
                //mexPrintf("%f %f\n",m1[(j*n)], v1[0]);
                for ( ulng j = 0; j < n; j++ )
                {
                    //mexPrintf("%f %f\n",m1[(j*m)+i], v1[j]);
                    returnVector[i] = alpha*m1[(j*m)+i]*v1[j];
                }
            }
        }
#endif
}

void Matrix::MVMultG(dbl *m1, dbl *v1, dbl *returnVector, 
        ulng m, ulng n, dbl alpha, dbl beta, char chn)
{
	/* Performs a general matrix vector mult y <- alpha*A*x + beta*y */
#ifdef BLAS
        MatrixVectorMultBLAS(m1, v1, returnVector, m, n, alpha, beta, chn);
#else
        if( chn == 'T' )
        {
            for ( ulng i = 0; i < n ; i++)
            {   
                returnVector[i] = alpha*RealInnerProduct(&m1[(i*m)],v1,m) +
					beta*returnVector[i] ;
            }
        }else{            
            for ( ulng i = 0; i < m ; i++)
            {
                //returnVector[i] = 0;
				returnVector[i] = beta*returnVector[i];
                //mexPrintf("%f %f\n",m1[(j*n)], v1[0]);
                for ( ulng j = 0; j < n; j++ )
                {
                    //mexPrintf("%f %f\n",m1[(j*m)+i], v1[j]);
                    returnVector[i] = beta*returnVector[i] + alpha*m1[(j*m)+i]*v1[j];
                }
            }
        }
#endif
}

void ZeroFillDbl(dbl *v, ulng n)
{
	/* Fills the dbl array with zeros by allocating
	zeros to every byte in of v */
	size_t num = static_cast<size_t>(n*sizeof(dbl));
	memset(v, 0, num );
}


dbl Matrix::VNorm(dbl *atom, ulng szSignal, ulng incx)
{
	#ifdef BLAS
				dbl normAtom = NormBLAS(atom,szSignal,incx);
	#else            
				dbl normAtom = sqrt(RealInnerProduct(atom,atom,szSignal,incx, incx));
	#endif
    return normAtom;    
}

dbl Matrix::VNormalize(dbl *atom, ulng nRows)
{   
	dbl normAtom = Matrix::VNorm(atom,nRows);

    Matrix::Scale(atom,1/normAtom,nRows);
        
    return normAtom;
}

void Matrix::Scale(dbl *v1, dbl alpha, ulng szVector)
{
    
#ifdef BLAS
        ScaleVectorBLAS(v1,alpha, szVector);
#else
        for ( ulng i = 0; i < szVector; i++)
        {
            v1[i] = v1[i]*alpha;
        }
#endif
}

dbl Matrix::RealInnerProduct(dbl *v1, dbl *v2, ulng szVector, 
        ulng incx, ulng incy)
{
    dbl innerProduct = 0;
#ifdef BLAS
        innerProduct = InnerProductBLAS(v1, v2, szVector, incx, incy);
#else
        if ((incx == 1) && (incy == 1))
        {
            for ( ulng i = 0; i < szVector; i++)
            {
                innerProduct += v1[i]*v2[i];  
            }
        }else if ((incx > 1) && (incy == 1))
        {
            for ( ulng i = 0; i < szVector; i++)
            {
                innerProduct += v1[i*incx]*v2[i];  
            }
        }else if ((incx == 1) && (incy > 1))
        {
            for ( ulng i = 0; i < szVector; i++)
            {
                innerProduct += v1[i]*v2[i*incy];  
            }
		}else if ((incx > 1) && (incy > 1))
        {
            for ( ulng i = 0; i < szVector; i++)
            {
                innerProduct += v1[i*incx]*v2[i*incy];  
            }
        }
#endif
    return innerProduct;
}

void Matrix::VAdd(dbl *v1, dbl *v2, dbl alpha, ulng szVector, 
        ulng incx, ulng incy)
{
    // v1 = v1 + alpha*v2
#ifdef BLAS
        AddVectorsBLAS(v2, v1, alpha, szVector, incy, incx);
#else
        for (ulng i = 0; i < szVector; i++ )
        {
            v1[i*incx] = alpha*v2[i*incy] + v1[i*incx];
        }
#endif
}

void Matrix::MMMult( double *m1, double *m2, double *mReturn, ulng m, ulng n, ulng k,
        ulng nRowsReturn,char chn1, char chn2,dbl alpha)
{
	/* Performs Matrix*Matrix*alpha */

	// m is the number of rows of m1
	// n is the number of columns of m1
	// k is the number of columns of mReturn
	#ifdef BLAS
		// Zero the return matrix
		ZeroFillDbl(mReturn,nRowsReturn*k);

		MatrixMatrixMultBLAS(m1, m2, mReturn, m, n, k, chn1, chn2,1,1);
	#else
	// Alpha and beta not applied
		if ( (chn1 == 'N') && (chn2 == 'N') )
		{
			for ( ulng i = 0; i < m; i ++ )
			{
				for ( ulng j = 0; j < k; j++ )
				{
					mReturn[(j*m)+i] = 0;
					for ( ulng l = 0; l < n; l++ )
					{
						mReturn[(j*m)+i] = mReturn[(j*m)+i] + m1[(l*m)+i]*m2[(j*n)+l];
					}
				}
			}
		}else if ( (chn1 == 'T') && (chn2 == 'N') )
		{
			for ( ulng i = 0; i < n; i ++ )
			{
				for ( ulng j = 0; j < k; j++ )
				{
					mReturn[(j*n)+i] = 0;
					for ( ulng l = 0; l < m; l++ )
					{
						mReturn[(j*n)+i] = mReturn[(j*n)+i] + m1[(i*m)+l]*m2[(j*m)+l];
					}
				}
			}
		}else if ( (chn1 == 'N') && (chn2 == 'T') )
		{
			for ( ulng i = 0; i < m; i ++ )
			{
				for ( ulng j = 0; j < k; j++ )
				{
					mReturn[(j*m)+i] = 0;
					for ( ulng l = 0; l < n; l++ )
					{
						mReturn[(j*m)+i] = mReturn[(j*m)+i] + m1[(l*m)+i]*m2[(l*k)+j];
					}
				}
			}
		}else if ( (chn1 == 'T') && (chn2 == 'T') )
		{
			for ( ulng i = 0; i < n; i ++ )
			{
				for ( ulng j = 0; j < k; j++ )
				{
					mReturn[(j*n)+i] = 0;
					for ( ulng l = 0; l < m; l++ )
					{
						mReturn[(j*n)+i] = mReturn[(j*n)+i] + m1[(i*m)+l]*m2[(j*m)+l];
					}
				}
			}
		}
	#endif
}


void Matrix::MMMultG( double *m1, double *m2, double *mReturn, ulng m, ulng n, ulng k,
        dbl alpha, dbl beta, char chn1, char chn2)
{
	/* Performs y<-alpha*A*B + beta*C */

	// m is the number of rows of m1
	// n is the number of columns of m1
	// k is the number of columns of mReturn
	#ifdef BLAS
		MatrixMatrixMultBLAS(m1, m2, mReturn, m, n, k, chn1, chn2,alpha,beta);
	#else
		if ( (chn1 == 'N') && (chn2 == 'N') )
		{
			for ( ulng i = 0; i < m; i ++ )
			{
				for ( ulng j = 0; j < k; j++ )
				{
					mReturn[(j*m)+i] = 0;
					for ( ulng l = 0; l < n; l++ )
					{
						mReturn[(j*m)+i] = beta*mReturn[(j*m)+i] + alpha*m1[(l*m)+i]*m2[(j*n)+l];
					}
				}
			}
		}else if ( (chn1 == 'T') && (chn2 == 'N') )
		{
			for ( ulng i = 0; i < n; i ++ )
			{
				for ( ulng j = 0; j < k; j++ )
				{
					mReturn[(j*n)+i] = 0;
					for ( ulng l = 0; l < m; l++ )
					{
						mReturn[(j*n)+i] = beta*mReturn[(j*n)+i] + alpha*m1[(i*m)+l]*m2[(j*m)+l];
					}
				}
			}
		}else if ( (chn1 == 'N') && (chn2 == 'T') )
		{
			for ( ulng i = 0; i < m; i ++ )
			{
				for ( ulng j = 0; j < k; j++ )
				{
					mReturn[(j*m)+i] = 0;
					for ( ulng l = 0; l < n; l++ )
					{
						mReturn[(j*m)+i] = beta*mReturn[(j*m)+i] + alpha*m1[(l*m)+i]*m2[(l*k)+j];
					}
				}
			}
		}else if ( (chn1 == 'T') && (chn2 == 'T') )
		{
			for ( ulng i = 0; i < n; i ++ )
			{
				for ( ulng j = 0; j < k; j++ )
				{
					mReturn[(j*n)+i] = 0;
					for ( ulng l = 0; l < m; l++ )
					{
						mReturn[(j*n)+i] = beta*mReturn[(j*n)+i] + alpha*m1[(i*m)+l]*m2[(j*m)+l];
					}
				}
			}
		}
	#endif
}


// void Matrix::VABS(double *v,ulng nElements)
// {
//     
//     /* Replaces each element of the vector v with its absolute value */
//     for( ulng i = 0; i < nElements; i++ )
//     {
//         v[i] = v[i] < 0 ? -v[i] : v[i];
//     }
// 
// }

// void Matrix::VABS(double *v,ulng nElements)
// {
//     
//     /* Replaces each element of the vector v with its absolute value */
//     for( ulng i = 0; i < nElements; i++ )
//     {        
//         v[i] = v[i] < 0.0 ? -v[i] : v[i];
//     }
// 
// }

//void Matrix::VABS(double *v,ulng nElements)
//{
//    
//    /* Replaces each element of the vector v with its absolute value */
//    for( int i = nElements-1; i >= 0; i-- )
//    {        
//        v[i] = v[i] < 0.0 ? -v[i] : v[i];
//    }
//
//}

void Matrix::VABS(double * v,ulng nElements)
{
    
    /* Replaces each element of the vector v with its absolute value */
    for( ulng i = 0 ; i < nElements; ++i )
    {        
        v[i] = v[i] < 0.0 ? -v[i] : v[i];
    }

}

unsigned long Matrix::MaxAbs(double *v, unsigned long m)
{
// Future add max loop
	// BLAS function returns index 1-n we want c array index 0-n-1
	return MaxAbsBLAS(v, m) - 1;

}