    #include "mex.h"
    #define msg mexPrintf
    #include "./Common/PursuitFunctions.h"
    //#include "./Common/MatrixOperations.h"
    #include <algorithm>

int HBW_OMP2D_call(mxArray *mApproxImageCM, const mxArray *mImageCM, double *mDictionaryY, double *mDictionaryX, double tol,const int nE,const unsigned long szSpaceX,const unsigned long szSpaceY, 
        unsigned long nAtomsX, unsigned long nAtomsY, mxArray *vIYBigCM, mxArray *vIXBigCM, mxArray *mBetaCM, mxArray *vCnCM, mxArray *mOrthogonalCM, 
           unsigned long No, int *kq, mxArray *nat_)
{
    double *nat = mxGetPr(nat_);
    nat[0]=0;
    
    const char name[] = "HBW_OMP2D";
        
    const unsigned long szSpaceYX = szSpaceY*szSpaceX;
    const unsigned long nAtomsYX = nAtomsY*nAtomsX;
    
    /* More validation */        
    unsigned long maxPickFromCell =  min(szSpaceYX,nAtomsYX); // max number atoms that can be chosen for cell 
    unsigned long nIterations = maxPickFromCell * nE; // max number of iterations for the whole image
    if (No < 0  || No > nIterations)
    {
        msg("The minimum number of atoms %i must be greater than 0 and less than %i!\n",No,nIterations);
        return 1;
    }else if (No != 0)
    {          
        nIterations = min(nIterations, No);
    }
    
    /* Check positive tol */
    if ( tol < 0 )
    {
        msg("Tol is %g is should be positive!\n",tol);
        return 1;
    }

    const double tol1 = 1e-7;  //atoms having smaller norm than tol1 are supposed be zero ones
    const double tol2 = 1e-10; //threshold for coefficients (whether to accept Ck, if makes a difference) 

    
    /* Re=f;*/
    mxArray *mResiduleCM = mxDuplicateArray(mImageCM); //intital residue == image

    unsigned int minCn = 0;
    
    
    /*************************************************************/
    /********************* Start of main routine *****************/
    /*************************************************************/

    msg("\n%s MEX is running for tol=%g, tol1=%g and tol2=%g.\n",name,tol,tol1,tol2);
    msg("tol  -> required precision, distance ||f-f_approx||\n");
    msg("tol1 -> atoms havin g smaller norm are supposed to be zero ones.\n");
    msg("tol2 -> algorithm stops if max(|<f,q>|/||q||)<= tol2.\n");
    
    /* Declare here as needed after the loop exits */
    unsigned long k = 0;
    double ssResidule = 0; //### perhaps need ssResidule for each block
    double *chosenAtom = new double[szSpaceYX]; //### might have to have all double[real] like this. CE [must be const szSpaceYX] -but are not set to zero
    for (int i=0; i<szSpaceYX; i++){
       chosenAtom[i] = 0;
    } 
	            
    mxArray *qq = mxCreateCellMatrix(nE,1); //store the indices of chosen dict index, each cell will have [q1 q2] array.
    mxArray *CpDM = mxCreateDoubleMatrix(1,nE,mxREAL); //store maximum value for each block
    double *Cp = mxGetPr(CpDM);
    mxArray *noreDM = mxCreateDoubleMatrix(1,nE,mxREAL); //store nore value for each block
    double *nore = mxGetPr(noreDM);

    //Calculate maxvalue for all the blocks.     
    //############ create store for indexs named indexCM. -we created earlier with notation qq
    double maxValue = 0;
    unsigned long iChosenAtomY = 0;
    unsigned long iChosenAtomX = 0;
    unsigned long  iChosenAtomXY = 0;

    for (int b=0; b<nE; b++){   
        double defineNoreb= 0; 
        mxArray *mImageCM_ = mxGetCell(mImageCM,b); 
        double *mImage = mxGetPr(mImageCM_);

        /* Choose the next atom from the unselected atoms. */
        /*  for p=k:N, %it can be done without for
         *      cc(p)=abs(Re*D(:,p));
         *  end      
         *  [max_c,q]=max(cc);      
         */    

        ChooseAtomOMP2D(mDictionaryY, mDictionaryX, mImage, nAtomsY, nAtomsX, szSpaceY, szSpaceX, maxValue, 
                iChosenAtomY, iChosenAtomX, iChosenAtomXY);  

        mxArray *indexs = mxCreateDoubleMatrix(2,1,mxREAL);
        double *setIndexs = mxGetPr(indexs);
        setIndexs[0]= iChosenAtomY; //*(setIndexs++) = iChosenAtomY;
        setIndexs[1]= iChosenAtomX; // *setIndexs = iChosenAtomX;       
        //mxSetCell(qq,b,setIndexs);//setting indexes CE
        mxSetCell(qq,b,indexs);// mxArray* 

        Cp[b]=maxValue;

        defineNoreb  = Norm(mImage, szSpaceYX);         
        defineNoreb = pow(defineNoreb,2);
        nore[b]= defineNoreb;
    } 


    double maxValueAllB=0;
    int qb=-1;  // will store index of chosen box
    
    for ( k = 0; k < nIterations; k++)
    {    
        qb= -1; //at the end of loop will reference the chosen block 
        maxValueAllB=0;
        for (int b=0; b<nE; b++){
            if (Cp[b]>maxValueAllB){
                maxValueAllB=Cp[b]; 
                qb= b; //this is the chosen block  
            }
        }

        /* Stopping criterion (coefficient)
         * MATLAB CODE
         * if max_c<tol2
         *      fprintf('%s stopped, max(|<f,q>|/||q||)<= tol2=%g.\n',name,tol2);
         *      break;
         * end
         */

        if ( qb == -1 || maxValueAllB < tol2 ) //No block was chosen, or have reached coefficient criterion
        {
            msg("\n \n %s stopped, max(|<f,q>|/||q||)<= tol2=%g.\n",name,tol2);
            break;    
        }

        //obtain indexes of the chosen b block
        mxArray *chosenIndexs_= mxGetCell(qq,qb);
        double *chosenIndexs = mxGetPr(chosenIndexs_);
        iChosenAtomY = (long) chosenIndexs[0]; //*(chosenIndexsVal++);
        iChosenAtomX = (long) chosenIndexs[1]; //*chosenIndexsVal;
        kq[qb]= kq[qb] + 1; // increase chosen Num iteration for chosen block
        nat[0]= nat[0] + 1; // increase total number of atoms count

        //~~ If we choose block qb for the first time, intialise oversized sroring arrays for that chosen block 
        if (kq[qb] == 1){
            //vIYBig
            mxArray *aCell_e1 = mxCreateDoubleMatrix(maxPickFromCell,1,mxREAL);
            mxSetCell(vIYBigCM,qb,aCell_e1);

            //vIXBig
            mxArray *aCell_e2 = mxCreateDoubleMatrix(maxPickFromCell,1,mxREAL);
            mxSetCell(vIXBigCM,qb,aCell_e2);

            //mBetaBig
            mxArray *aCell_e3 = mxCreateDoubleMatrix(szSpaceYX , maxPickFromCell, mxREAL);
            mxSetCell(mBetaCM,qb,aCell_e3);

            mxArray *aCell_e4 = mxCreateDoubleMatrix(szSpaceYX , maxPickFromCell, mxREAL);
            mxSetCell(mOrthogonalCM,qb,aCell_e4);

            //vCoefficients
            mxArray *aCell_e5 = mxCreateDoubleMatrix(maxPickFromCell,1,mxREAL);
            mxSetCell(vCnCM,qb,aCell_e5);
        }        
        //~~


        /* Populate the matrices with the new atoms
         * MATLAB CODE
         *    Di1(k)=q(1);
         *    Di2(k)=q(2);
         */
        /* Add 1 as MATLAB index's */
        //vIXBig[k] = iChosenAtomX + 1; 
        mxArray *vIXBig_ = mxGetCell(vIXBigCM,qb);
        double *vIXBig = mxGetPr(vIXBig_);
        vIXBig[ (kq[qb])-1  ]  = iChosenAtomX + 1;

        mxArray *vIYBig_ = mxGetCell(vIYBigCM,qb);
        double *vIYBig = mxGetPr(vIYBig_);
        vIYBig[ (kq[qb])-1  ]  = iChosenAtomY + 1;


        /* Get the chosen atom */
        KronAtom(&mDictionaryX[iChosenAtomX*szSpaceX],&mDictionaryY[iChosenAtomY*szSpaceY],szSpaceX,szSpaceY,chosenAtom);


        //Get  Orthogonal 'Block' of the chosen block qb
        mxArray *mOrthogonal_ = mxGetCell(mOrthogonalCM,qb); //pointer to cell 1 of 6
        double *mOrthogonal = mxGetPr(mOrthogonal_);
        //Get  Beta 'Block' of the chosen block qb
        mxArray *mBeta_ = mxGetCell(mBetaCM,qb); //pointer to cell 1 of 6
        double *mBeta = mxGetPr(mBeta_);
        //Get Residule 'Block' of the chosen block qb
        mxArray *mResidule_ = mxGetCell(mResiduleCM,qb);
        double *mResidule = mxGetPr(mResidule_);
        //Get Image 'Block' of the chosen block qb
        mxArray *mImageCM_ = mxGetCell(mImageCM,qb); //pointer to cell 1 of 6
        double *mImage = mxGetPr(mImageCM_);

        unsigned long iNewAtom = ((kq[qb])-1)*szSpaceYX; //so we know which column to orthogonalise, ie second row if we have chosen 2 atoms.
        //we do minus 1 so we get back to c++ indexing. ie 0 to (szSpaceYX-1)


        /*  if k==1 
         *      Q(:,k)=D(:,k); 
         *  end
         */
        if ( kq[qb] == 1) //means, first we chose from block qb   
        {
            CopyElements( chosenAtom, mOrthogonal, szSpaceYX );
        }

        /* if k>1 */
        if ( kq[qb] > 1)
        {
            /*  for p=1:k-1
             *      Q(:,k)=D(:,k)-(Q(:,p)'*D(:,k))*Q(:,p); %orthogonalization
             *  end 
             */
            OrthogonalizeOMP(mOrthogonal, chosenAtom, szSpaceYX, kq[qb]-1);

            /* Re-orthogonalization of Q(:,k)  w.r.t Q(:,1),..., Q(:,k-1)
             * MATLAB CODE
             *  for zi=1:zmax      
             *      for p=1:k-1
             *          Q(:,k)=Q(:,k)-(Q(:,p)'*Q(:,k))*Q(:,p);
             *   	end	   
             *  end
             */     
            Reorthognalize(mOrthogonal,szSpaceYX,kq[qb]-1,2);
        }              


        /* Normalize atom Q(:,k)
         * MATLAB CODE
         * nork=norm(Q(:,k));
         * Q(:,k)=Q(:,k)/nork; %normalization
         */      
        double normKthAtom = Normalize(&mOrthogonal[iNewAtom],szSpaceYX,0);

        /* Compute biorthogonal functions beta
         *   if k>1
         *      beta=beta-Q(:,k)*(D(:,k)'*beta)/nork; 
         *  end	
         *  beta(:,k)=Q(:,k)/nork;          % kth biorthogonal function 
         */
        CalcBiorthogonal(mBeta, chosenAtom, &mOrthogonal[iNewAtom], normKthAtom, szSpaceYX, kq[qb]-1);

        /* %%%%computes de residum%%%%%    
         * h=f(:)'*Q(:,k)*Q(:,k)';
         * Re=Re(:)-h';
         * nore=(norm(Re))^2*(delta);
         * Re=reshape(Re,Lx,Ly);        
         */
        /* Works the same as in 1D as all matrices are stored as 1D arrays */
        //CalcResiduleOMP(mResidule, mImage, &mOrthogonal[k*szSpaceYX], szSpaceYX);       
        CalcResiduleOMP(mResidule, mImage, &mOrthogonal[iNewAtom], szSpaceYX);       


        /* Calculate the norm of the residule */
        /*  MATLAB CODE
         *  Re = norm(f'-D(:,1:k)*(f*beta)')
         */  
        ssResidule  = Norm(mResidule, szSpaceYX);         
        ssResidule = pow(ssResidule,2);
        nore[qb]= ssResidule;

        double max_err=0;

        for ( int i=0; i<nE; i++){
            max_err = max_err + nore[i]; //sum nore 
        }

        /* Sum Squared Error */
        //normResidule = sqrt(RealInnerProduct( vResidule, vResidule, szSignal ));

        /* Stopping criteria (distance)
         * MATLAB CODE - We use the residule check with omp if this is the same
         * if (norm(f'-D(:,1:k)*(f*beta)')*sqrt(delta) < tol) && (tol~= 0)break;end;
         */
        // if ( ssResidule < tol && tol != 0 && k+1 > minCn)
        if ( max_err < tol && tol != 0 ) // nat[o] > minCn
        {
            /* Break so will not increment k before exiting the loop */
            // k += 1;
            //  
            mexPrintf("\n \n We have have reached (tol) for approximation. tol= %f, max_err= %f", tol,max_err);
            break;
         }

         //recalculate max value for the chosen block. obvs the chosen block has changed so look at residule 
         ChooseAtomOMP2D(mDictionaryY, mDictionaryX, mResidule, nAtomsY, nAtomsX, szSpaceY, szSpaceX, maxValue,iChosenAtomY, iChosenAtomX, iChosenAtomXY);  
         mxArray *indexs_rcal = mxGetCell(qq,qb);
         double *setIndexs_rcal = mxGetPr(indexs_rcal);
         setIndexs_rcal[0]= iChosenAtomY; //*(setIndexs++) = iChosenAtomY;
         setIndexs_rcal[1]= iChosenAtomX; //*setIndexs = iChosenAtomX;       
         Cp[qb]=maxValue;

    }
 
    /** End of iteration **/
    
    int ic;       
    for (ic =0; ic<nE; ic++){
        if (kq[ic] != 0){

            /* c=f*beta; */

            //Get  Beta 'Block' of the block i
            mxArray *mBeta_el = mxGetCell(mBetaCM,ic); //pointer to cell i of nE
            double *mBetael = mxGetPr(mBeta_el);
            //Get  Image 'Block' of the block i
            mxArray *mImage_el = mxGetCell(mImageCM,ic); //pointer to cell i of nE
            double *mImageel = mxGetPr(mImage_el);  
            //Get  vCn 'Block' of the block i
            mxArray *vCn_el = mxGetCell(vCnCM,ic); //pointer to cell i of nE
            double *vCnel = mxGetPr(vCn_el);

            //mexPutVariable("base", "CellBlockBeta", mBeta_);
            //mexPutVariable("base", "CellBlockmImage", mImage_);
            //mexPutVariable("base", "CellBlockvCn", vCn_);


            MultiplyMatrixVector(mBetael, mImageel, vCnel, szSpaceYX, kq[ic], 'T');


            /* MATLAB Code
             *  for n=1:id;
             *      H=H+Dx2(:,n)*c(n)*Dy2(:,n)';
             *  end
             * Just subtract the residule for speed otherwise would be quicker to store all
             * the chosen atoms and multiply them by mImage make a function so can be altered easily.
             */ 

            /* mResidule = -mResidule + mImage */
                // AddVectors1(mResidule, mImage, -1, szSpaceYX);
            //Get Residule 'Block' of the chosen block i
            mxArray *mResidule_el = mxGetCell(mResiduleCM,ic);
            double *mResiduleel = mxGetPr(mResidule_el);
            AddVectors1(mResiduleel, mImageel, -1, szSpaceYX);


            /* mApproxImage = mResidule */
            // CopyElements(mResidule,mApproxImage,szSpaceYX);
            mxArray *mApproxImage_el = mxGetCell(mApproxImageCM,ic);
            double *mApproxImageel = mxGetPr(mApproxImage_el);
            CopyElements(mResiduleel,mApproxImageel,szSpaceYX);

        }
    }
    
    return 0;
}

